//------------------------------------------------------------------------------
// GUI for Extract dialog
//
// ToDo: - MipMaps
//       - S3TC
//
// (c) 2000, 2001, Jesco Topp & Andy Ford
//------------------------------------------------------------------------------
unit App_Extract;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, UTPFF_Load, UTPFF_Aux_Functions,
  App_Extract_Thread;

type
  TDlg_Extract = class(TForm)
    CancelBtn: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label3: TLabel;
    PkgCount: TLabel;
    TexCount: TLabel;
    GroupBox2: TGroupBox;
    Extract_MipMaps: TCheckBox;
    Preserve_Groups: TCheckBox;
    Btn_Start: TButton;
    GroupBox3: TGroupBox;
    ComPrg: TProgressBar;
    CurPrg: TProgressBar;
    Label4: TLabel;
    Label23: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    CurPkg: TLabel;
    CurTex: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    lblTime: TLabel;
    lblTimeLeft: TLabel;
    Panel1: TPanel;
    Image1: TImage;
    txtExtractTo: TStaticText;
    procedure Btn_StartClick(Sender: TObject);
    function  ShowModal : Integer; override;
    procedure CancelBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    // Extract Modes
    procedure Single;
    procedure Complete;
    { Private declarations }
  public
    FileName    : string;                        // path & name of texture package
    TextureId   : integer;                       // index of texture into ExportTable (only
                                                 // if single texture will be exported)
    FileList    : array[0..512] of string;       // list of all packages including pathes
                                                 // (only if all packages will be extracted)
    FileCount   : integer;                       // count of packages in list
                                                 // (only if all packages will be extracted)
    ExtractMode : BYTE;                          // 1: package
                                                 // 2: complete
  end;

var
  Dlg_Extract      : TDlg_Extract;
  ExtractThread    : TExtractThread;
  InProgress       : Boolean;

implementation

uses Main, App_Preferences;

{$R *.DFM}

//------------------------------------------------------------------------------
// init dialog
//
function TDlg_Extract.ShowModal;
var
  UTX_File      : TUTPackage_Load;   // temporary object to get texture count
  i             : integer;
  tmp_TexCount  : integer;
begin
  // alter caption of dialog
  case ExtractMode of
  1: begin
       PkgCount.Caption := '1';

       UTX_File := TUTPackage_Load.Create;
       UTX_File.Load( FileName );
       TexCount.Caption := IntToStr( UTX_File.TextureList.Count );
       UTX_File.Free;
     end;
  2: begin
       PkgCount.Caption := IntToStr( FileCount );

       tmp_TexCount := 0;
       for i:=0 to FileCount-1 do
       begin
         UTX_File := TUTPackage_Load.Create;
         UTX_File.Load( FileList[i] );
         tmp_TexCount := tmp_TexCount + UTX_File.TextureList.Count;
         UTX_File.Free;
       end;

       TexCount.Caption := IntToStr( tmp_TexCount );
     end;
  end;

  // clear from previous run
  ComPrg.Position := 0;
  CurPrg.Position := 0;

  CurPkg.Caption := ' ';
  CurTex.Caption := ' ';

  CancelBtn.Caption := '&Cancel';
  InProgress := FALSE;

  ExtractThread.Free;

  Result := inherited ShowModal;
end;

//------------------------------------------------------------------------------
// extract textures
//

// user started extraction-process
procedure TDlg_Extract.Btn_StartClick(Sender: TObject);
begin
  // check mode
  case ExtractMode of
    1 : begin
          Single;
        end;
    2 : begin
          Complete;
        end;
  end;
end;

// extract single package
procedure TDlg_Extract.Single;
begin
  InProgress := TRUE;

  ComPrg.Min := 0;
  ComPrg.Max := StrToInt( TexCount.Caption );
  ComPrg.Step := 1;
  ComPrg.Position := 0;

  ExtractThread := TExtractThread.Create(FALSE);

  // logging
  MainForm.App_Log( 'Extracting ' + PkgCount.Caption + ' package ( ' + TexCount.Caption + ' Textures )', ICO_INFO );
end;

// extract complete package-list
procedure TDlg_Extract.Complete;
begin
  InProgress := TRUE;

  ComPrg.Min := 0;
  ComPrg.Max := StrToInt( TexCount.Caption );
  ComPrg.Step := 1;
  ComPrg.Position := 0;

  ExtractThread := TExtractThread.Create(FALSE);

  // logging
  MainForm.App_Log( 'Extracting ' + PkgCount.Caption + ' packages ( ' + TexCount.Caption + ' Textures )', ICO_INFO );
end;

procedure TDlg_Extract.CancelBtnClick(Sender: TObject);
begin
  if ( InProgress ) then
    ExtractThread.Terminate;
end;

// Show extraction path
procedure TDlg_Extract.FormShow(Sender: TObject);
begin
//  txtExtractTo.Caption := 'To : ' + Dlg_Preferences.SaveDir.Caption;
end;

end.
