//------------------------------------------------------------------------------
// TPalette_Load : base-class for loading palettes
//
// (c) 2000, 2001, Jesco Topp & Andy Ford
//------------------------------------------------------------------------------
unit UTPFF_Load_Palette;

interface

uses Windows,
     UTPFF_Aux_Classes, UTPFF_Aux_Functions;

//------------------------------------------------------------------------------
type
  TRGBQuad = record
    r       : BYTE;
    g       : BYTE;
    b       : BYTE;
    a       : BYTE;
end;

//------------------------------------------------------------------------------
type
  TPalette_Load = Class(TObject)
    Palette_NAME   : string;                     // Name of the palette
    Palette_INDEX  : DWORD;                      // ObjRef
    Palette_ETINDEX: DWORD;                      // Index into ExportTable
    PaletteSize    : DWORD;                      // Number of entries
    PaletteOffset  : DWORD;                      // Offset
    Color          : array [0..255] of TRGBQuad; // unconverted palette-entries
    WinPalette     : HPalette;                   // converted HPALETTE
    // copy of tables
    NameTable   : TNameTable;
    ImportTable : TImportTable;
    ExportTable : TExportTable;
    constructor Create;
    procedure Free;
    destructor Destroy; override;
  public
    //Loads the under Name specified palette and converts it to a HPALETTE
    procedure Load(var NT : TNameTable; IT : TImportTable; ET : TExportTable;
                   var F : File; INDEX : DWORD);            //Reads and converts the specified palette
  private
    procedure   Read_Palette(var F : File);
    procedure   Convert_RAW2HPALETTE;
end;

//------------------------------------------------------------------------------
implementation

//------------------------------------------------------------------------------
constructor TPalette_Load.Create;
begin
end;

destructor TPalette_Load.Destroy;
begin
  inherited Destroy;
end;

procedure TPalette_Load.Free;
begin
  if ( Self <> nil ) then Destroy;
end;

// loads specified palette
procedure TPalette_Load.Load;
begin
  // copy tables
  NameTable   := NT;
  ImportTable := IT;
  ExportTable := ET;
  Palette_INDEX := INDEX;

  // Read raw data
  Read_Palette( F );

  // Convert to HPalette
  Convert_RAW2HPALETTE;
end;

procedure TPalette_Load.Read_Palette;
var
  PaletteOffset  : longint;
  i              : integer;
begin
  // get offset of palette in current file
  Palette_INDEX := Palette_INDEX - 1;               // palette is always in exporttable
  PaletteOffset := ExportTable.SerialOffset[Palette_INDEX];

  Seek(F, PaletteOffset);

  // read property (only a 'None' one should exist)
  Read_INDEX(F);

  // read data
  PaletteSize := Read_INDEX(F);
  for i:=0 to PaletteSize-1 do
  begin
    BlockRead(F, Color[i].r, 1);
    BlockRead(F, Color[i].g, 1);
    BlockRead(F, Color[i].b, 1);
    BlockRead(F, Color[i].a, 1);
  end;      
end;

//Create Hpalette
procedure TPalette_Load.Convert_RAW2HPALETTE;
var
  pal     : PLogPalette;
  i       : integer;
begin
  pal := nil;
  try
    GetMem(pal, sizeof(TLogPalette) + sizeof(TPaletteEntry) * PaletteSize);
    pal.palVersion := $300;
    pal.palNumEntries := PaletteSize;
    for i := 0 to PaletteSize-1 do
    begin
      pal.palPalEntry[i].peRed := Color[i].r;
      pal.palPalEntry[i].peGreen := Color[i].g;
      pal.palPalEntry[i].peBlue := Color[i].b;
      pal.palPalEntry[i].peFlags := 0;
    end;
    WinPalette := CreatePalette(pal^);
  finally
    FreeMem(pal);
  end;
end;

end.
